#version 120

/* SHADOWRES:1024 */
/* SHADOWHPL:75.0 */
#define SHADOWRES 512
#define SHADOWHPL 75.0

uniform sampler2D shadow;
uniform sampler2D watershadow;

/* GAUX4FORMAT:RGBA32F */
/* GNORMALFORMAT:RGB32F */

uniform vec3 cameraPosition;
uniform vec3 sunPosition;
uniform vec3 moonPosition;
uniform int fogMode;
uniform int worldTime;
uniform int isEyeInWater;
uniform float rainStrength;
uniform float wetness;
uniform float far;
uniform float near;
uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowProjectionInverse;
uniform mat4 shadowModelView;
uniform mat4 shadowModelViewInverse;
uniform float viewWidth;
uniform float viewHeight;
uniform float aspectRatio;
uniform float sunAngle;
uniform float moonAngle;
const int GL_LINEAR = 9729;
const int GL_EXP = 2048;
varying vec4 texcoord;
varying vec3 lightVector;
varying float lightAngle;
uniform sampler2D gcolor;
uniform sampler2D gdepth;
uniform sampler2D gaux1;
uniform sampler2D gaux3;
uniform sampler2D gaux4;
uniform sampler2D gnormal;
uniform sampler2D composite;
uniform sampler2D depthtex0;
vec3 nvec3(vec4 pos){
    return pos.xyz/pos.w;
}
vec4 nvec4(vec3 pos){
    return vec4(pos.xyz, 1.0);
}
float cdist(vec2 coord){
    return max(abs(coord.s - 0.5), abs(coord.t - 0.5)) * 2.0;
}
float average(vec3 color){
    return (color.r + color.g + color.b)/3;
}
vec3 tocs(vec3 spos){
    return nvec3(gbufferProjectionInverse * nvec4(spos * 2.0 - 1.0));
}
vec3 toss(vec3 cpos){
    return nvec3(gbufferProjection * nvec4(cpos)) * 0.5 + 0.5;
}
vec3 tows(vec3 cpos){
    return nvec3(gbufferModelViewInverse * nvec4(cpos));
}
vec3 s_tocs(vec3 wpos){
    return nvec3(shadowModelView * nvec4(wpos));
}
vec3 s_toss(vec3 s_cpos){
    return nvec3(shadowProjection * nvec4(s_cpos)) * 0.5 + 0.5;
}
vec3 n_coord(vec3 normal){
    return normalize(normal * 2.0 - 1.0);
}
vec3 n_color(vec3 normal){
    return clamp(normal * 0.5 + 0.5, 0.0, 1.0);
}
vec3 s_ss(vec3 spos){
  return s_toss(s_tocs(tows(tocs(spos))));
}
vec3 s_cs(vec3 spos){
  return s_tocs(tows(tocs(spos)));
}

vec2 unpack(float v){
    float m = floor(v * 10);
    float d = (v * 10 - m);
    return vec2(m, d);
}

vec2 unlmp(float value){
    if(value < 1.0){
        float l = floor(value*100)/100;
        float m = (value - l)*100;
        return vec2(l,m);
    }
    return vec2(1.0);
}
  
vec4 getPixel(vec2 coord){/*for post proc*/
    return texture2D(gaux3, coord.st);
}

vec4 getColor(vec2 coord){
    return texture2D(gcolor, coord.st);
}

vec4 getWater(vec2 coord){
    return texture2D(composite, coord.st);
}
  
vec4 getGDepth(vec2 coord){
    vec3 raw = texture2D(gdepth, coord.st).rgb;
    return vec4(raw.rgb / raw.b, 1.0);
}
  
float getDepth(vec2 coord){
    float depth = getGDepth(coord.st).r;
    if(depth < 1){
        return unpack(depth).t;
    } else {
        return 1.0;
    }
}

float getDepthWater(vec2 coord){
    return texture2D(depthtex0, coord.st).r;
}
  
vec2 getLmCoord(vec2 coord){
    return unlmp(getGDepth(coord.st).g);
}
  
vec2 getLmCoordWater(vec2 coord){
    float alpha = sqrt(texture2D(gaux4, coord.st).a);
    if(alpha > 0.1){
        return unlmp((alpha - 0.11) / 0.89);
    } else {
        return getLmCoord(coord.st);
    }
    return vec2(1.0);
}

vec4 getNormal(vec2 coord){
    return texture2D(gnormal, coord.st);
}

vec4 getNormalWater(vec2 coord){
    float alpha = sqrt(texture2D(gaux4, coord.st).a);
    if(alpha > 0.0){
        return vec4(texture2D(gaux4, coord.st).rgb / alpha, 1.0);
    } else {
        return texture2D(gnormal, coord.st);
    }
}

float checkSky(vec2 coord){
    if(getGDepth(coord.st).r > .99) return 1;
    return 0;
}
  
float checkSkyWater(vec2 coord){
    if(texture2D(gaux1, coord.st).r < 0.01) return 1;
    return 0;
}



float getWaves(vec3 worldposition){
   float wsize = 0.1f*1.5;
   float wspeed = 0.3f;

   float rs0 = abs(sin((worldTime*wspeed/5.0) + (worldposition.s*wsize) * 20.0 + (worldposition.z*4.0))+0.2);
   float rs1 = abs(sin((worldTime*wspeed/7.0) + (worldposition.t*wsize) * 27.0) + 0.5);
   float rs2 = abs(sin((worldTime*wspeed/2.0) + (worldposition.t*wsize) * 60.0 - sin(worldposition.s*wsize) * 13.0)+0.4);
   float rs3 = abs(sin((worldTime*wspeed/1.0) - (worldposition.s*wsize) * 20.0 + cos(worldposition.t*wsize) * 83.0)+0.1);

   float wsize2 = 0.05f*0.75;
   float wspeed2 = 0.2f;

   float rs0a = abs(sin((worldTime*wspeed2/4.0) + (worldposition.s*wsize2) * 24.0) + 0.5);
   float rs1a = abs(sin((worldTime*wspeed2/11.0) + (worldposition.t*wsize2) * 77.0  - (worldposition.z*6.0)) + 0.5);
   float rs2a = abs(sin((worldTime*wspeed2/6.0) + (worldposition.s*wsize2) * 50.0 - (worldposition.t*wsize2) * 23.0) + 0.5);
   float rs3a = abs(sin((worldTime*wspeed2/14.0) - (worldposition.t*wsize2) * 4.0 + (worldposition.s*wsize2) * 98.0) + 0.5);

   float wsize3 = 0.01f*0.125;
   float wspeed3 = 0.3f;

   float rs0b = abs(sin((worldTime*wspeed3/4.0) + (worldposition.s*wsize3) * 14.0) + 0.5);
   float rs1b = abs(sin((worldTime*wspeed3/11.0) + (worldposition.t*wsize3) * 37.0 + (worldposition.z*1.0)) + 0.5);
   float rs2b = abs(sin((worldTime*wspeed3/6.0) + (worldposition.t*wsize3) * 47.0 - cos(worldposition.s*wsize3) * 33.0 + rs0a + rs0b) + 0.5);
   float rs3b = abs(sin((worldTime*wspeed3/14.0) - (worldposition.s*wsize3) * 13.0 + sin(worldposition.t*wsize3) * 98.0 + rs0 + rs1) + 0.5);

   float waves = (rs1 * rs0 + rs2 * rs3)/2.0f;
   float waves2 = (rs0a * rs1a + rs2a * rs3a)/2.0f;
   float waves3 = (rs0b + rs1b + rs2b + rs3b)/4.0f;
   return (waves + waves2 + waves3)/3.0f;
}

float getWavesC(vec3 worldposition){
   float wsize = 9.0f*3.0;
   float wspeed = 0.3f;

   float rs0 = abs(sin((worldTime*wspeed/5.0) + (worldposition.s*wsize) * 20.0 + (worldposition.z*4.0))+0.2);
   float rs1 = abs(sin((worldTime*wspeed/7.0) + (worldposition.t*wsize) * 27.0) + 0.5);
   float rs2 = abs(sin((worldTime*wspeed/2.0) + (worldposition.t*wsize) * 60.0 - sin(worldposition.s*wsize) * 13.0)+0.4);
   float rs3 = abs(sin((worldTime*wspeed/1.0) - (worldposition.s*wsize) * 20.0 + cos(worldposition.t*wsize) * 83.0)+0.1);

   float wsize2 = 5.4f*1.5;
   float wspeed2 = 0.2f;

   float rs0a = abs(sin((worldTime*wspeed2/4.0) + (worldposition.s*wsize2) * 24.0) + 0.5);
   float rs1a = abs(sin((worldTime*wspeed2/11.0) + (worldposition.t*wsize2) * 77.0  - (worldposition.z*6.0)) + 0.5);
   float rs2a = abs(sin((worldTime*wspeed2/6.0) + (worldposition.s*wsize2) * 50.0 - (worldposition.t*wsize2) * 23.0) + 0.5);
   float rs3a = abs(sin((worldTime*wspeed2/14.0) - (worldposition.t*wsize2) * 4.0 + (worldposition.s*wsize2) * 98.0) + 0.5);

   float wsize3 = 2.0f*0.75;
   float wspeed3 = 0.3f;

   float rs0b = abs(sin((worldTime*wspeed3/4.0) + (worldposition.s*wsize3) * 14.0) + 0.5);
   float rs1b = abs(sin((worldTime*wspeed3/11.0) + (worldposition.t*wsize3) * 37.0 + (worldposition.z*1.0)) + 0.5);
   float rs2b = abs(sin((worldTime*wspeed3/6.0) + (worldposition.t*wsize3) * 47.0 - cos(worldposition.s*wsize3) * 33.0 + rs0a + rs0b) + 0.5);
   float rs3b = abs(sin((worldTime*wspeed3/14.0) - (worldposition.s*wsize3) * 13.0 + sin(worldposition.t*wsize3) * 98.0 + rs0 + rs1) + 0.5);

   float waves = (rs1 * rs0 + rs2 * rs3)/2.0f;
   float waves2 = (rs0a * rs1a + rs2a * rs3a)/2.0f;
   float waves3 = (rs0b + rs1b + rs2b + rs3b)/4.0f;
   return (waves + waves2)/2.0f;
}

float getShadows(vec3 fragpos){
   vec3 worldposition = tows(fragpos);
   float dist = sqrt(fragpos.x * fragpos.x + fragpos.y * fragpos.y + fragpos.z * fragpos.z);
   if (dist < SHADOWHPL && dist > 0.1) {
      float xzDistanceSquared = worldposition.x * worldposition.x + worldposition.z * worldposition.z;
      float yDistanceSquared  = worldposition.y * worldposition.y;
      if (yDistanceSquared < (SHADOWHPL*SHADOWHPL*0.75)) {
         worldposition = s_tocs(worldposition);
         float comparedepth = -worldposition.z;
         worldposition = s_toss(worldposition);
         if (comparedepth > 0.0 && worldposition.s < 1.0 && worldposition.s > 0.0 && worldposition.t < 1.0 && worldposition.t > 0.0){
            float shadowMult = clamp(1.0 - xzDistanceSquared / (SHADOWHPL*SHADOWHPL*1.0), 0.0, 1.0) * clamp(1.0 - yDistanceSquared / (SHADOWHPL*SHADOWHPL*1.0), 0.0, 1.0);
            float sampleDistance = 0.25 / SHADOWRES;
            float shadowSample = texture2D(shadow, worldposition.st).z;
            float shadowDepth = 0.05 + shadowSample * (256.0 - 0.05);
            return 1.0 - shadowMult * clamp(comparedepth - shadowDepth - 0.1, 0.0, 1.0);
         }
      }
   }
   return 1.0;
}

float getShadowsAnWater(vec3 fragpos){
   vec3 worldposition = tows(fragpos);
   float dist = sqrt(fragpos.x * fragpos.x + fragpos.y * fragpos.y + fragpos.z * fragpos.z);
   if (dist < SHADOWHPL && dist > 0.1) {
      float xzDistanceSquared = worldposition.x * worldposition.x + worldposition.z * worldposition.z;
      float yDistanceSquared  = worldposition.y * worldposition.y;
      if (yDistanceSquared < (SHADOWHPL*SHADOWHPL*0.75)) {
         worldposition = s_tocs(worldposition);
         float comparedepth = -worldposition.z;
         worldposition = s_toss(worldposition);
         if (comparedepth > 0.0 && worldposition.s < 1.0 && worldposition.s > 0.0 && worldposition.t < 1.0 && worldposition.t > 0.0){
            float shadowMult = clamp(1.0 - xzDistanceSquared / (SHADOWHPL*SHADOWHPL*1.0), 0.0, 1.0) * clamp(1.0 - yDistanceSquared / (SHADOWHPL*SHADOWHPL*1.0), 0.0, 1.0);
            float sampleDistance = 0.25 / SHADOWRES;
            float shadowSample = texture2D(watershadow, worldposition.st).z;
            float shadowDepth = 0.05 + shadowSample * (256.0 - 0.05);
            return 1.0 - shadowMult * clamp(comparedepth - shadowDepth - 0.1, 0.0, 1.0);
         }
      }
   }
   return 1.0;
}

void main(){
    vec4 color = getColor(texcoord.st);
    vec4 water = getWater(texcoord.st);
    vec3 coord = vec3(texcoord.st, getDepth(texcoord.st));
    vec3 wcoord = vec3(texcoord.st, getDepthWater(texcoord.st));
    vec3 normal = n_coord(getNormal(texcoord.st).rgb);
    vec3 wnormal = n_coord(getNormalWater(texcoord.st).rgb);
    vec3 wfragpos = tocs(wcoord.xyz);
    vec3 fragpos = tocs(coord.xyz);
    float pix_x = 1.0/viewWidth;
    float pix_y = 1.0/viewHeight;
    if(dot(normal, normalize(fragpos)) > 0){
        normal *= -1;
    }
    if(dot(wnormal, normalize(wfragpos)) > 0){
        wnormal *= -1;
    }

    if(water.a > 0.0){
      color.rgb /= 0.89;
    }

    //DEFAULT
    float shading  = 1.0f;
    float wshading = 1.0f;
    float xshading = 0.0f;

    //SPECULAR
    if(checkSky(coord.st) < 0.01) {
      shading = getShadows(fragpos.xyz);
    }
    if(checkSkyWater(coord.st) < 0.01 && abs(unpack(texture2D(gaux1, coord.st).r).s-6)>=1) {
      wshading = getShadows(wfragpos.xyz);
      xshading = wshading * abs(sin(sunAngle*(2*acos(-1.0))));
      //xshading = clamp(xshading, 0.0, 1.0);
    }

    //GROUND
    if(checkSky(coord.st) < 0.01) {
      vec2 lm = getLmCoord(coord.st);
      if(shading > 0.0){
        if(dot(getNormal(coord.st).rgb, vec3(1.0)) > 0.01){
          shading *= 1.0 - dot(normalize(-lightVector), normalize(normal));
        }
        shading *= 1.0 - abs(sin(abs(getWavesC(s_ss(coord.xyz)))*acos(-1.0))) * (1.0 - getShadowsAnWater(fragpos.xyz));
        shading = clamp(shading, 0.0, 1.0);
      }
      shading = clamp(1.0 - (1.0 - shading) * abs(sin(sunAngle*(2*acos(-1.0)))), 0.0, 1.0);
      shading = clamp(1.0 - (1.0 - shading) * lm.t, 0.0, 1.0);
      shading = max(shading, lm.s);
      shading = clamp(shading, 0.4, 1.0);
      shading = mix(shading, 0.9, clamp(rainStrength, 0.0, 1.0));
      color.rgb *= shading;
    }

    //WATER
    if(checkSkyWater(coord.st) < 0.01 && abs(unpack(texture2D(gaux1, coord.st).r).s-6)>=1 && abs(unpack(texture2D(gaux1, texcoord.st).r).s-4)>=1){
      vec2 wlm = getLmCoordWater(wcoord.st);
      if(wshading > 0.0){
        if(dot(getNormalWater(coord.st).rgb, vec3(1.0)) > 0.01){
          wshading *= 1.0 - dot(normalize(-lightVector), normalize(wnormal));
        }
        wshading *= 1.0 - abs(sin(abs(getWavesC(s_ss(wcoord.xyz)))*acos(-1.0))) * (1.0 - getShadowsAnWater(wfragpos.xyz));
        wshading = clamp(wshading, 0.0, 1.0);
      }
      wshading = clamp(1.0 - (1.0 - wshading) * abs(sin(sunAngle*(2*acos(-1.0)))), 0.0, 1.0);
      wshading = clamp(1.0 - (1.0 - wshading) * wlm.t, 0.0, 1.0);
      wshading = max(wshading, wlm.s);
      wshading = clamp(wshading, 0.4, 1.0);
      wshading = mix(wshading, 0.9, clamp(rainStrength, 0.0, 1.0));
      water.rgb *= wshading;
    }

    //gl_FragColor = color;
    gl_FragData[0] = clamp(vec4(color.rgb, 1.0), 0.0, 1.0);
    gl_FragData[1] = texture2D(gdepth, texcoord.st);
    gl_FragData[2] = texture2D(gnormal, texcoord.st);
    gl_FragData[3] = clamp(water, 0.0, 1.0);
    gl_FragData[4] = vec4(texture2D(gaux1, texcoord.st).r, xshading, getWaves(s_cs(wcoord.xyz)), 1.0);
    gl_FragData[6] = texture2D(gaux3, texcoord.st);
    gl_FragData[7] = texture2D(gaux4, texcoord.st);
}