#version 120

#define REFRACTIONS
#define REFLECTIONS
#define REFLECT_SKY
#define SPECULAR

//Reflection Config
const float ext = 0.66f;
const float stp = 0.001;
const float ref = 0.1f;
const float inc = 2.0f;
const int maxf = 12;

/* GAUX4FORMAT:RGBA32F */
/* GNORMALFORMAT:RGB32F */

uniform vec3 cameraPosition;
uniform vec3 sunPosition;
uniform vec3 moonPosition;
uniform int fogMode;
uniform int worldTime;
uniform int isEyeInWater;
uniform float rainStrength;
uniform float wetness;
uniform float far;
uniform float near;
uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowProjectionInverse;
uniform mat4 shadowModelView;
uniform mat4 shadowModelViewInverse;
uniform float viewWidth;
uniform float viewHeight;
uniform float aspectRatio;
uniform float sunAngle;
uniform float moonAngle;
const int GL_LINEAR = 9729;
const int GL_EXP = 2048;
varying vec4 texcoord;
varying vec3 lightVector;
varying float lightAngle;
uniform sampler2D gcolor;
uniform sampler2D gdepth;
uniform sampler2D gaux1;
uniform sampler2D gaux3;
uniform sampler2D gaux4;
uniform sampler2D gnormal;
uniform sampler2D composite;
uniform sampler2D depthtex0;
vec3 nvec3(vec4 pos){
    return pos.xyz/pos.w;
}
vec4 nvec4(vec3 pos){
    return vec4(pos.xyz, 1.0);
}
float cdist(vec2 coord){
    return max(abs(coord.s - 0.5), abs(coord.t - 0.5)) * 2.0;
}
float average(vec3 color){
    return (color.r + color.g + color.b)/3;
}
vec3 tocs(vec3 spos){
    return nvec3(gbufferProjectionInverse * nvec4(spos * 2.0 - 1.0));
}
vec3 toss(vec3 cpos){
    return nvec3(gbufferProjection * nvec4(cpos)) * 0.5 + 0.5;
}
vec3 tows(vec3 cpos){
    return nvec3(gbufferModelViewInverse * nvec4(cpos));
}
vec3 s_tocs(vec3 wpos){
    return nvec3(shadowModelView * nvec4(wpos));
}
vec3 s_toss(vec3 s_cpos){
    return nvec3(shadowProjection * nvec4(s_cpos)) * 0.5 + 0.5;
}
vec3 n_coord(vec3 normal){
    return normalize(normal * 2.0 - 1.0);
}
vec3 n_color(vec3 normal){
    return clamp(normal * 0.5 + 0.5, 0.0, 1.0);
}
vec3 s_ss(vec3 spos){
  return s_toss(s_tocs(tows(tocs(spos))));
}
vec3 s_cs(vec3 spos){
  return s_tocs(tows(tocs(spos)));
}

vec2 unpack(float v){
    float m = floor(v * 10);
    float d = (v * 10 - m);
    return vec2(m, d);
}

vec2 unlmp(float value){
    if(value < 1.0){
        float l = floor(value*100)/100;
        float m = (value - l)*100;
        return vec2(l,m);
    }
    return vec2(1.0);
}
  
vec4 getPixel(vec2 coord){/*for post proc*/
    return texture2D(gaux3, coord.st);
}

vec4 getColor(vec2 coord){
    return texture2D(gcolor, coord.st);
}

vec4 getWater(vec2 coord){
    return texture2D(composite, coord.st);
}
  
vec4 getGDepth(vec2 coord){
    vec3 raw = texture2D(gdepth, coord.st).rgb;
    return vec4(raw.rgb / raw.b, 1.0);
}
  
float getDepth(vec2 coord){
    float depth = getGDepth(coord.st).r;
    if(depth < 1){
        return unpack(depth).t;
    } else {
        return 1.0;
    }
}

float getDepthWater(vec2 coord){
    return texture2D(depthtex0, coord.st).r;
}
  
vec2 getLmCoord(vec2 coord){
    return unlmp(getGDepth(coord.st).g);
}
  
vec2 getLmCoordWater(vec2 coord){
    float alpha = sqrt(texture2D(gaux4, coord.st).a);
    if(alpha > 0.1){
        return unlmp((alpha - 0.11) / 0.89);
    } else {
        return getLmCoord(coord.st);
    }
    return vec2(1.0);
}

vec4 getNormal(vec2 coord){
    return texture2D(gnormal, coord.st);
}

vec4 getNormalWater(vec2 coord){
    float alpha = sqrt(texture2D(gaux4, coord.st).a);
    if(alpha > 0.0){
        return vec4(texture2D(gaux4, coord.st).rgb / alpha, 1.0);
    } else {
        return texture2D(gnormal, coord.st);
    }
}

float checkSky(vec2 coord){
    if(getGDepth(coord.st).r > .99) return 1;
    return 0;
}
  
float checkSkyWater(vec2 coord){
    if(texture2D(gaux1, coord.st).r < 0.01) return 1;
    return 0;
}



float getFog(vec2 coord){
  float fog = 0.0f;
  vec3 pos = tocs(vec3(coord.st, getDepth(coord.st)));
  if(checkSky(coord.st) < 0.01){
    float dist = sqrt(pos.x*pos.x + pos.y*pos.y + pos.z*pos.z);
    if (fogMode == GL_EXP) {
      fog = 1.0 - clamp(exp(-gl_Fog.density * dist), 0.0, 1.0);
    } else if (fogMode == GL_LINEAR) {
      fog = clamp((dist - gl_Fog.start) * gl_Fog.scale, 0.0, 1.0);
    }
  }
  return fog;
}

float getFogWater(vec2 coord){
  float fog = 0.0f;
  vec3 pos = tocs(vec3(coord.st, getDepthWater(coord.st)));
  if(checkSkyWater(coord.st) < 0.01){
    float dist = sqrt(pos.x*pos.x + pos.y*pos.y + pos.z*pos.z);
    if (fogMode == GL_EXP) {
      fog = 1.0 - clamp(exp(-gl_Fog.density * dist), 0.0, 1.0);
    } else if (fogMode == GL_LINEAR) {
      fog = clamp((dist - gl_Fog.start) * gl_Fog.scale, 0.0, 1.0);
    }
  }
  return fog;
}

float getWaves(vec2 coord){
  return texture2D(gaux1, coord.st).b;
}

vec4 getReflected(vec3 fragpos, vec3 normal, vec3 bcolor){
    if(dot(normal, normalize(fragpos)) >= 0){
      normal *= -1;
    }
    float fade_rate = clamp(1 + dot(normalize(fragpos), normalize(normal)), 0.0, 1.0);
    float fade = mix(0.4, 1.0, fade_rate);
    vec4 color = vec4(0.0);
    vec3 vector = stp * normalize(reflect(normalize(fragpos), normalize(normal)));
    vec3 oldpos = fragpos /*+ vector*/;
    vec3 opos = toss(fragpos);
    fragpos = oldpos + vector;
    int sr = 0;
    int it = 0;
    int br = 0;
    while(it < 60){
        vec3 pos = toss(fragpos);
        if(pos.x < 0 || pos.x > 1 || pos.y < 0 || pos.y > 1 || pos.z < 0 || pos.z > 1.1) { break; }
        vec3 spos = vec3(pos.st, getDepth(pos.st));
        spos = tocs(spos);
        float diff = abs(fragpos.z - spos.z);
        float dist = distance(fragpos, spos);
        vec4 raw = getNormal(pos.st);
        float orientation = dot(normalize(raw.xyz * 2.0 - 1.0), vector) * -dot(raw.xyz * 2.0 - 1.0, normalize(fragpos));
        float len = length(vector);
        float err = max(pow(len, ext), len);
        if(err - min(diff, dist) >= 0 && (orientation <= 0 || dot(raw.rgb, vec3(1.0)) < 0.01 || raw.a < 0.01) && getColor(pos.st).a > 0.0){
            sr++;
            fragpos = oldpos;
            vector *= ref;
        }
        if(sr > maxf){
            color = vec4(mix(getColor(pos.st).rgb, gl_Fog.color.rgb, getFog(pos.st)), clamp(1.0 - pow(cdist(pos.st), 8.0), 0.0, 1.0));
            break;
        }
        vector *= inc;
        oldpos = fragpos;
        fragpos += vector;
        it++;
    }
#ifdef REFLECT_SKY
    color.rgb = mix(mix(vec3(0.0), gl_Fog.color.rgb, clamp(pow(getLmCoordWater(opos.st).t * 16.0, 2.0) - 1.5, 0.0, 1.0)), color.rgb, color.a);
    color.a = 1.0;
#endif
    color.a *= fade;
    return color;
}

vec4 getRefracted(vec3 fragpos, vec3 normal){
    if(dot(normal, normalize(fragpos)) >= 0){
      normal *= -1;
    }
    vec3 vector = normalize(refract(normalize(fragpos), normalize(normal), 0.8));
    vec3 pos = toss(fragpos + vector);
    vec4 color = vec4(mix(getColor(pos.st).rgb, gl_Fog.color.rgb, getFog(pos.st)), 1.0);
    if(getWater(pos.st).a < 0.01 || pos.x < 0 || pos.x > 1 || pos.y < 0 || pos.y > 1){
        color.a = 0.0;
    }
    return color;
}
vec2 getRefractedCoord(vec3 fragpos, vec3 normal){
    if(dot(normal, normalize(fragpos)) >= 0){
      normal *= -1;
    }
    vec3 vector = normalize(refract(normalize(fragpos), normalize(normal), 0.8));
    vec3 pos = toss(fragpos + vector);
    if(getWater(pos.st).a < 0.01 || pos.x < 0 || pos.x > 1 || pos.y < 0 || pos.y > 1){
        return texcoord.st;
    }
    return pos.st;
}

void main(){
  float timefract = worldTime;
  float timePow = 3.0f;
  float TimeSunrise  = ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0) + (1.0 - (clamp(timefract, 0.0, 6000.0)/6000.0));
  float TimeNoon     = ((clamp(timefract, 0.0, 6000.0)) / 6000.0) - ((clamp(timefract, 6000.0, 12000.0) - 6000.0) / 6000.0);
  float TimeSunset   = ((clamp(timefract, 6000.0, 12000.0) - 6000.0) / 6000.0) - ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0);
  float TimeMidnight = ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0) - ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0);

  //Curve times
  TimeSunrise  = pow(TimeSunrise, timePow);
  TimeNoon     = pow(TimeNoon, 1.0f/timePow);
  TimeSunset   = pow(TimeSunset, timePow);
  TimeMidnight = pow(TimeMidnight, 1.0f/timePow);

  const float rayleigh = 0.0f;
  
  vec3 sunrise_sun;
   sunrise_sun.r = 1.0 * TimeSunrise;
   sunrise_sun.g = 0.46 * TimeSunrise;
   sunrise_sun.b = 0.00 * TimeSunrise;
  
  vec3 sunrise_amb;
   sunrise_amb.r = 0.00 * TimeSunrise;
   sunrise_amb.g = 0.23 * TimeSunrise;
   sunrise_amb.b = 0.999 * TimeSunrise;  
   
  vec3 noon_sun;
   noon_sun.r = mix(1.00, 1.00, rayleigh) * TimeNoon;
   noon_sun.g = mix(1.00, 0.48, rayleigh) * TimeNoon;
   noon_sun.b = mix(0.98, 0.00, rayleigh) * TimeNoon;  
  
  vec3 noon_amb;
   noon_amb.r = 0.00 * TimeNoon * 1.0;
   noon_amb.g = 0.18 * TimeNoon * 1.0;
   noon_amb.b = 0.999 * TimeNoon * 1.0;
  
  vec3 sunset_sun;
   sunset_sun.r = 1.0 * TimeSunset;
   sunset_sun.g = 0.38 * TimeSunset;
   sunset_sun.b = 0.0 * TimeSunset;
  
  vec3 sunset_amb;
   sunset_amb.r = 0.252 * TimeSunset;
   sunset_amb.g = 0.427 * TimeSunset;
   sunset_amb.b = 0.999 * TimeSunset;
  
  vec3 midnight_sun;
   midnight_sun.r = 0.45 * 0.8 * 0.325 * TimeMidnight;
   midnight_sun.g = 0.6 * 0.8 * 0.325 * TimeMidnight;
   midnight_sun.b = 0.8 * 0.8 * 0.325 * TimeMidnight;
  
  vec3 midnight_amb;
   midnight_amb.r = 0.3 * 0.001 * TimeMidnight;
   midnight_amb.g = 0.4 * 0.001 * TimeMidnight;
   midnight_amb.b = 0.8 * 0.001 * TimeMidnight;

  vec3 sunlight_color;
   sunlight_color.r = sunrise_sun.r + noon_sun.r + sunset_sun.r + midnight_sun.r;
   sunlight_color.g = sunrise_sun.g + noon_sun.g + sunset_sun.g + midnight_sun.g;
   sunlight_color.b = sunrise_sun.b + noon_sun.b + sunset_sun.b + midnight_sun.b;
  
  vec3 ambient_color;
   ambient_color.r = sunrise_amb.r + noon_amb.r + sunset_amb.r + midnight_amb.r;
   ambient_color.g = sunrise_amb.g + noon_amb.g + sunset_amb.g + midnight_amb.g;
   ambient_color.b = sunrise_amb.b + noon_amb.b + sunset_amb.b + midnight_amb.b;
   
  vec3 reflected_color;
   reflected_color = mix(sunlight_color, ambient_color, 0.5f);

  vec3 ambfill_color;
   ambfill_color = mix(sunlight_color, ambient_color, 0.55f);
   
  vec3 skycolor = mix(sunlight_color, vec3(1.0f), 0.5f);
   
   float sun_fill = 0.251f * (1.0f - TimeMidnight);
  
   ambient_color = mix(ambient_color, sunlight_color, sun_fill);
   vec3 ambient_color_rain = vec3(0.09, 0.09, 0.09) * (1.0f - TimeMidnight * 1.0f); //rain
   ambient_color = mix(ambient_color, ambient_color_rain, clamp(rainStrength, 0.0, 1.0)); //rain
   
   ambient_color = pow(ambient_color, vec3(0.454545f));
   sunlight_color = pow(sunlight_color, vec3(1.454545f));
   reflected_color = pow(reflected_color, vec3(0.454545f));
   ambfill_color = pow(ambfill_color, vec3(0.454545f));
    vec4 color = getColor(texcoord.st);
    vec4 water = getWater(texcoord.st);
    vec3 coord = vec3(texcoord.st, getDepth(texcoord.st));
    vec3 wcoord = vec3(texcoord.st, getDepthWater(texcoord.st));
    vec3 normal = n_coord(getNormal(texcoord.st).rgb);
    vec3 wnormal = n_coord(getNormalWater(texcoord.st).rgb);
    vec3 wfragpos = tocs(wcoord.xyz);
    vec3 fragpos = tocs(coord.xyz);
    float pix_x = 1.0/viewWidth;
    float pix_y = 1.0/viewHeight;
    if(dot(normal, normalize(fragpos)) > 0){
        normal *= -1;
    }
    if(dot(wnormal, normalize(wfragpos)) > 0){
        wnormal *= -1;
    }

    float wfresnel = 1.0;
    float waterSpec = 0.0;
    float spec = 0.0;
    float totalspec = clamp(unpack(texture2D(gaux1, coord.st).r).t * (1 - isEyeInWater), 0.0, 1.0);
    if(abs(unpack(texture2D(gaux1, texcoord.st).r).s-4)<1){
        float rspread = 0.30f;
        float rdepth = wcoord.z;
        float waves = getWaves(texcoord.st);
            waves -= 0.5f;
            waves *= 1.0 - rdepth;
            waves *= 100.0f;

        float wavesdeltax1 =  getWaves(texcoord.st) - getWaves(texcoord.st + vec2(-pix_x, 0.0f));
        float wavesdeltax2 = -getWaves(texcoord.st) + getWaves(texcoord.st + vec2( pix_x, 0.0f));
        float wavesdeltax = 0.0f;
        if(abs(wavesdeltax1) > abs(wavesdeltax2)){
            wavesdeltax = wavesdeltax2;
        } else {
            wavesdeltax = wavesdeltax1;
        }
        wavesdeltax *= 1.0f - rdepth;
        wavesdeltax *= 30.0f;
        wavesdeltax = clamp(wavesdeltax, -0.1f, 0.1f);
            
        float wavesdeltay1 =  getWaves(texcoord.st) - getWaves(texcoord.st + vec2(0.0f, -pix_y));
        float wavesdeltay2 = -getWaves(texcoord.st) + getWaves(texcoord.st + vec2(0.0f,  pix_y));
        float wavesdeltay = 0.0f;
        if(abs(wavesdeltay1) > abs(wavesdeltay2)){
            wavesdeltay = wavesdeltay2;
        } else {
            wavesdeltay = wavesdeltay1;
        }
        wavesdeltay *= 1.0f - rdepth;
        wavesdeltay *= 30.0f;
        wavesdeltay = clamp(wavesdeltay, -0.1f, 0.1f);

        const float wnormalclamp = 0.05f;

        float wnormal_x1 = getDepthWater(texcoord.st + vec2(pix_x, 0.0f)) - getDepthWater(texcoord.st);
        float wnormal_x2 = getDepthWater(texcoord.st) - getDepthWater(texcoord.st + vec2(-pix_x, 0.0f));      
        float wnormal_x = 0.0f;
        if(abs(wnormal_x1) > abs(wnormal_x2)){
          wnormal_x = wnormal_x2;
        } else {
          wnormal_x = wnormal_x1;
        }
        wnormal_x /= 1.0f - rdepth; 
        wnormal_x = clamp(wnormal_x, -wnormalclamp, wnormalclamp);
        wnormal_x *= rspread*1.0f;

        float wnormal_y1 = getDepthWater(texcoord.st + vec2(0.0f, pix_y*1.0f)) - getDepthWater(texcoord.st);
        float wnormal_y2 = getDepthWater(texcoord.st) - getDepthWater(texcoord.st + vec2(0.0f, -pix_y));   
        float wnormal_y;
        if(abs(wnormal_y1) > abs(wnormal_y2)){
          wnormal_y = wnormal_y2;
        } else {
          wnormal_y = wnormal_y1;
        } 
        wnormal_y /= 1.0f - rdepth;     
        wnormal_y = clamp(wnormal_y, -wnormalclamp, wnormalclamp);
        wnormal_y *= rspread*1.0f*aspectRatio;

        wnormal -= vec3(vec2(wavesdeltax, wavesdeltay) * 5.0, 0.0);
        wnormal = normalize(wnormal);

        wfresnel = pow(distance(vec2(wnormal_x, wnormal_y) + vec2(wavesdeltax, wavesdeltay) * 0.1, vec2(0.0f)), 0.7f) * 15.0f;

        float waterRoughness = clamp(1.0f - getDepthWater(texcoord.st), 0.0f, 1.0f);
        float waterRoughness2 = clamp(getDepthWater(texcoord.st), 0.0f, 1.0f);
        vec3 wnormalsLocal = normalize(wnormal + vec3(wavesdeltax, wavesdeltay, 0.0f) * (40.0f * waterRoughness2));
        vec3 halfVector = normalize(lightVector - normalize(tocs(wcoord).xyz));

        waterSpec = max(0.0f, dot(halfVector, wnormalsLocal));
        waterSpec = pow(waterSpec, 111151.0f * waterRoughness + 50.0f);
        waterSpec = max(waterSpec, 0.0);

        waterSpec *= 5.01f * (wfresnel + 0.2);
        spec = waterSpec;
    } else {
        spec = totalspec;
    }
    spec *= mix(1.0f, 0.0f, clamp(rainStrength, 0.0, 1.0));
    spec *= texture2D(gaux1, wcoord.st).g;
    spec = clamp(spec, 0.0, 1.0);

    vec2 rcoord = texcoord.st;
    if(sqrt(water.a) > 0){
        color.rgb = mix(color.rgb, gl_Fog.color.rgb, getFog(coord.st));
#ifdef REFRACTIONS
        if(checkSkyWater(coord.st) < 0.01 && abs(unpack(texture2D(gaux1, coord.st).r).s-6)>1 && dot(getNormalWater(coord.st).rgb, vec3(1.0)) > 0.01){
          if(isEyeInWater > 0) rcoord = getRefractedCoord(tocs(wcoord), wnormal);
          vec4 refr = getRefracted(tocs(wcoord), wnormal);
          color.rgb = mix(color.rgb, refr.rgb, refr.a);
        }
#endif
        color.rgb = water.rgb + (1.0 - sqrt(water.a)) * color.rgb;
    }

    if(totalspec > 0.1 && checkSkyWater(coord.st) < 0.01 && abs(unpack(texture2D(gaux1, coord.st).r).s-6)>=1 && dot(getNormalWater(coord.st).rgb, vec3(1.0)) > 0.01){
#ifdef SPECULAR
        float det = clamp(dot(lightVector, reflect(normalize(tocs(wcoord)), wnormal)), 0.0, 1.0);
        color.rgb += clamp(pow(det, 4) * spec * sunlight_color * (vec3(1.0) - color.rgb), 0.0, 1.0);
        color.rgb = clamp(color.rgb, 0.0, 1.0);
#endif
#ifdef REFLECTIONS
        vec4 refl = getReflected(tocs(wcoord), wnormal, ambient_color);
        color.rgb += refl.rgb * refl.a * (vec3(1.0) - color.rgb) * totalspec * 0.9;
#endif
    }

    if(texture2D(gdepth, rcoord.st).b < 1.0){
        color.rgb = mix(color.rgb, vec3(0.0), 0.2);
    }

    color.rgb = mix(color.rgb, gl_Fog.color.rgb, getFogWater(texcoord.st));

    gl_FragColor = clamp(vec4(vec3(color.rgb), 1.0), 0.0, 1.0);
}