#version 120

#define POM
#define NORMAL_MAP_MAX_ANGLE 0.5f

uniform vec3 cameraPosition;
uniform vec3 sunPosition;
uniform vec3 moonPosition;
uniform int fogMode;
uniform int worldTime;
uniform int isEyeInWater;
uniform float rainStrength;
uniform float wetness;
uniform float far;
uniform float near;
uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowProjectionInverse;
uniform mat4 shadowModelView;
uniform mat4 shadowModelViewInverse;
uniform float viewWidth;
uniform float viewHeight;
uniform float aspectRatio;
uniform float sunAngle;
uniform float moonAngle;
const int GL_LINEAR = 9729;
const int GL_EXP = 2048;
uniform sampler2D texture;
uniform sampler2D normals;
uniform sampler2D specular;
uniform sampler2D lightmap;
varying vec4 texcoord;
varying vec4 lmcoord;
varying vec3 tangent;
varying vec3 normal;
varying vec3 binormal;
varying vec3 viewVector;
varying vec4 color;
varying float distance;
varying float iswater;
varying float sprite;
varying float test;
const int linear_steps = 18;
const int binary_steps = 6;
const float bump_distance = 60.0;
const float pom_distance = 30.0;
const float fademult = 0.1;
const float texinterval = 0.0625;
const float pom_depth = 0.1;
float pack(float m, float d){
   return clamp(ceil(m)/10 + clamp(d, 0.0, 0.9999f)*.1, 0.0, 0.99999f);
}
float fad(float val){
    return clamp(floor(val*100.0)*.01, 0.0, .99);
}
float lmp(vec2 lmc){
    return fad(lmc.s) + fad(lmc.t)*.01;
}

void main(){
    gl_FragDepth = gl_FragCoord.z;
    vec3 coord = vec3(texcoord.st, 1.0);

mat3 tbn = mat3(tangent.x, binormal.x, normal.x,
                tangent.y, binormal.y, normal.y,
                tangent.z, binormal.z, normal.z);
if (normalize(tbn * viewVector).z < 0.0 && distance < pom_distance && texture2D(normals, coord.st).a < 1.0 && sprite < 0.5) {
    vec2 coordBase = coord.st - mod(coord.st, vec2(texinterval));
    vec3 coordWork = vec3(coord.st - coordBase, 1.0);
    coordWork.st /= texinterval;
    vec3 eview = normalize(viewVector.xyz);
    vec3 tview = normalize(vec3(dot(eview, tangent), dot(eview, binormal), dot(eview, -normalize(normal))));
    vec2 ds = tview.xy * pom_depth / tview.z;
    vec2 dp = coordWork.st;

    float depth_step = 1.0 / linear_steps;
    float size = depth_step;
    float depth = 1.0;
    float best_depth = 1.0;
    for (int i = 0 ; i < linear_steps - 1 ; ++i) {
      depth -= size;
      if (depth >= 1.0 - texture2D(normals, coordBase + mod(dp + ds * depth, 1.0) * texinterval).a)
        best_depth = depth;
    }
    depth = best_depth - size;
    for (int i = 0 ; i < binary_steps ; ++i) {
      size *= 0.5;
      if (depth >= 1.0 - texture2D(normals, coordBase + mod(dp + ds * depth, 1.0) * texinterval).a) {
        best_depth = depth;
        depth -= 2 * size;
      }
      depth += size;
    }
    
    coordWork.st = mod(dp + ds * best_depth, 1.0) * texinterval;
    coord.st = coordBase + coordWork.st;

    float dr = pom_depth * 10;
    float pomdepth = texture2D(normals, coord.st).a * dr - dr;
    float pomdepthbias = max(-(pomdepth) * (1.0 - gl_FragCoord.z) * (1.0 - gl_FragCoord.z), 0.0);
    gl_FragDepth = gl_FragCoord.z + pomdepthbias;
}

    vec4 colored = texture2D(texture, coord.st)* color;
    colored.rgb *= texture2D(lightmap, lmcoord.st).rgb;

    vec3 frag = normal * 0.5 + 0.5;
    vec3 bump = texture2D(normals, coord.st).xyz;
    float det = dot(bump, vec3(1.0));
    if(det < 0.01){
        bump = vec3(0.5, 0.5, 1.0);
    }
    bump = normalize(bump * 2.0 - 1.0);
    if (distance < bump_distance) {
        float bumpmult = clamp(bump_distance * fademult - distance * fademult, 0.0f, 1.0f) * NORMAL_MAP_MAX_ANGLE;
        frag = normalize((bump * vec3(bumpmult, bumpmult, bumpmult) + vec3(0.0f, 0.0f, 1.0f - bumpmult)) * tbn) * 0.5 + 0.5;
    }
  vec3 specularity = texture2D(specular,coord.xy).rgb;
  float wetx = clamp(wetness, 0.0, 1.0);
  float g_spec = specularity.r + specularity.g* wetx;
  float g_irr = specularity.b;
  float totalspec = (g_spec + g_irr) * 0.333 * mix(min(lmcoord.t+0.1,1.0),1.0,lmcoord.s);

    gl_FragData[0] = colored;
    gl_FragData[1] = vec4(pack(0, gl_FragDepth), lmp(lmcoord.st), 1.0, 1.0);
    gl_FragData[3] = gl_FragData[7] = vec4(0.0);
    if(sprite < 0.5){
        gl_FragData[2] = vec4(frag, 1.0);
    } else {
        gl_FragData[2] = vec4(normal * 0.5 + 0.5, 1.0);
    }
    if(sprite < 0.5){
        gl_FragData[4] = vec4(pack(2, totalspec), 0.0, 0.0, 1.0);
    } else {
        gl_FragData[4] = vec4(pack(2, 0.0), 0.0, 0.0, 1.0);
    }
}   